from django.conf import settings

FACEBOOK_APPLICATION_ID = getattr(settings, 'FACEBOOK_APPLICATION_ID')
FACEBOOK_APPLICATION_SECRET_KEY = getattr(settings, 'FACEBOOK_APPLICATION_SECRET_KEY')
FACEBOOK_APPLICATION_URL = getattr(settings, 'FACEBOOK_APPLICATION_URL')
DISABLED_PATHS = getattr(settings, 'FANDJANGO_DISABLED_PATHS', [])
ENABLED_PATHS = getattr(settings, 'FANDJANGO_ENABLED_PATHS', [])
